# Sage version information for Python scripts
# This file is auto-generated by the update-version script, do not edit!
version = '10.7.rc1'
date = '2025-08-05'
banner = 'SageMath version 10.7.rc1, Release Date: 2025-08-05'
